@@ -68,4 +68,11 @@ sub sqlt_deploy_hook {
   }
 }
 
+sub store_column {
+  my ($self, $name, $value) = @_;
+  $value = 'X '.$value if ($name eq 'name' && $value && $value =~ /store_column test/);
+  $self->next::method($name, $value);
+}
+
+
 1;
